Ray ID Specification
============================================
```

 high bits                                                                           low bits
<--------------------------------------------------------------------------------------------

                                                                                   4B
                                                                          +-----------------+
                                                                          |   unique bytes  |  JobID     4B
                                                                          +-----------------+

                                                                 4B                4B
                                                        +-----------------+-----------------+
                                                        |   unique bytes  |       JobID     |  ActorID   8B
                                                        +-----------------+-----------------+

                                        6B                               8B
                            +---------------------------+-----------------------------------+
                            |       unique bytes        |               ActorID             |  TaskID   14B
                            +---------------------------+-----------------------------------+

       4B            2B                                  14B
+---------------------------+---------------------------------------------------------------+
|  index bytes  |flags bytes|                             TaskID                            |  ObjectID 20B
+---------------------------+---------------------------------------------------------------+

```
#### JobID (4 bytes)
`JobID` is generated by `GCS` to ensure uniqueness. Its length is 4 bytes.

#### ActorID (8 bytes)
An `ActorID` contains two parts: 1) 4 unique bytes, and 2) its `JobID`.

#### TaskID (14 bytes)
A `TaskID` contains two parts: 1) 6 unique bytes, and 2) its `ActorID`.  
If the task is a normal task or a driver task, the part 2 is its dummy actor id.

The following table shows the layouts of all kinds of task id.
```
+-------------------+-----------------+------------+---------------------------+-----------------+
|                   | Normal Task     | Actor Task | Actor Creation Task       | Driver Task     |
+-------------------+-----------------+------------+---------------------------+-----------------+
| task unique bytes | random          | random     | nil                       | nil             |
+-------------------+-----------------+------------+---------------------------+-----------------+
| actor id          | dummy actor id* | actor id   | Id of the actor to create | dummy actor id* |
+-------------------+-----------------+------------+---------------------------+-----------------+
Note: Dummy actor id is an `ActorID` whose unique part is nil.
```

#### ObjectID (20 bytes)
An `ObjectID` contains 3 parts:
- `index bytes`: 4 bytes to indicate the index of the object.
- `flags bytes`: 2 bytes to indicate the flags of this object. We have 3 flags now: `created_by_task`, `object_type` and `transport_type`.
- `TaskID`: 14 bytes to indicate the ID of the task to which this object belongs.

**flags bytes format**
```
  1b     1b        3b                          11b
+-------------------------------------------------------------------------+
| (1) | (2) |     (3)      |                (4)unused                     |
+-------------------------------------------------------------------------+
```
- The (1) `created_by_task` part is one bit to indicate whether this `ObjectID` is generated (put or returned) from a task.

- The (2) `object_type` part is one bit to indicate the type of this object, whether a `PUT_OBJECT` or a `RETURN_OBJECT`.
    - `PUT_OBJECT` indicates this object is generated through `ray.put` during the task's execution.
    - `RETURN_OBJECT` indicates this object is the return value of a task.

- The (3) `transport_type` part is 3 bits to indicate the type of the transport which is used to transfer this object. So it can support 8 types.

- There are 11 bits unused in `flags bytes`.
